@@ -32,6 +32,8 @@ extern "C"
 
 using namespace WhiskerMenu;
 
+bool Plugin::m_menu_shown = false;
+
 //-----------------------------------------------------------------------------
 
 extern "C" void whiskermenu_construct(XfcePanelPlugin* plugin)
@@ -47,31 +49,28 @@ static void plugin_free(XfcePanelPlugin*, gpointer user_data)
 
 // Wait for grab; allows modifier as shortcut
 // Adapted from http://git.xfce.org/xfce/xfce4-panel/tree/common/panel-utils.c#n122
-static bool can_grab()
+static bool can_grab(GtkWidget* widget)
 {
-	GdkWindow* window = gdk_screen_get_root_window(xfce_gdk_screen_get_active(nullptr));
+	GdkWindow* window = gtk_widget_get_window(widget);
 	GdkDisplay* display = gdk_window_get_display(window);
 	GdkSeat* seat = gdk_display_get_default_seat(display);
-	GdkDevice* keyboard = gdk_seat_get_keyboard(seat);
 
-	// Don't try to get the grab for longer then 1/4 second
-	for (int i = 0; i < 2500; ++i)
+	for (int i = 0; i < 5; ++i)
 	{
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-		const GdkGrabStatus grab_status = gdk_device_grab(keyboard,
+		const GdkGrabStatus grab_status = gdk_seat_grab(seat,
 				window,
-				GDK_OWNERSHIP_NONE,
-				true,
-				GdkEventMask(GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK),
+				GDK_SEAT_CAPABILITY_ALL,
+				false,
 				nullptr,
-				GDK_CURRENT_TIME);
+				nullptr,
+				nullptr,
+				nullptr);
 		if (grab_status == GDK_GRAB_SUCCESS)
 		{
-			gdk_device_ungrab(keyboard, GDK_CURRENT_TIME);
+			gdk_seat_ungrab(seat);
 			return true;
 		}
-G_GNUC_END_IGNORE_DEPRECATIONS
-		g_usleep(100);
+		g_usleep(100000);
 	}
 
 	return false;
@@ -95,8 +94,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	m_plugin(plugin),
 	m_window(nullptr),
 	m_opacity(100),
-	m_file_icon(false),
-	m_focus_out_time(0)
+	m_file_icon(false)
 {
 	// Load settings
 	wm_settings = new Settings;
@@ -320,6 +318,7 @@ void Plugin::button_toggled(GtkToggleButton* button)
 	{
 		if (gtk_widget_get_visible(m_window->get_widget()))
 		{
+			m_menu_shown = false;
 			m_window->hide();
 		}
 		xfce_panel_plugin_block_autohide(m_plugin, false);
@@ -380,23 +379,19 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 		return false;
 	}
 
-	// Ignore event if menu lost focus and hid within last 1/4 second;
+	// Ignore event if last shown through remote event;
 	// needed for toggling as remote event happens after focus is lost
-	if (m_focus_out_time)
+	if (m_menu_shown && !wm_settings->stay_on_focus_out)
 	{
-		if ((g_get_monotonic_time() - m_focus_out_time) < 250000)
-		{
-			m_focus_out_time = 0;
-			return true;
-		}
-		m_focus_out_time = 0;
+		m_menu_shown = false;
+		return true;
 	}
 
 	if (gtk_widget_get_visible(m_window->get_widget()))
 	{
 		m_window->hide();
 	}
-	else if (!can_grab())
+	else if (!can_grab(gtk_widget_get_toplevel(m_button)))
 	{
 		g_printerr("xfce4-whiskermenu-plugin: Unable to get keyboard. Menu popup failed.\n");
 	}
@@ -467,6 +462,10 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 	}
 #if LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
 	gint icon_size = xfce_panel_plugin_get_icon_size(m_plugin);
+	if (!wm_settings->button_single_row)
+	{
+		icon_size *= xfce_panel_plugin_get_nrows(m_plugin);
+	}
 #else
 	GtkBorder padding, border;
 	GtkStyleContext* context = gtk_widget_get_style_context(m_button);
@@ -573,7 +572,7 @@ void Plugin::show_menu(bool at_cursor)
 		m_opacity = wm_settings->menu_opacity;
 	}
 	m_window->show(at_cursor ? Window::PositionAtCursor : Window::Position(xfce_panel_plugin_get_orientation(m_plugin)));
-	m_focus_out_time = 0;
+	m_menu_shown = true;
 }
 
 //-----------------------------------------------------------------------------
